# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.apps.meet_v2beta import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object]  # type: ignore

from google.protobuf import field_mask_pb2  # type: ignore

from google.apps.meet_v2beta.types import resource, service

from .client import SpacesServiceClient
from .transports.base import DEFAULT_CLIENT_INFO, SpacesServiceTransport
from .transports.grpc_asyncio import SpacesServiceGrpcAsyncIOTransport


class SpacesServiceAsyncClient:
    """REST API for services dealing with spaces."""

    _client: SpacesServiceClient

    DEFAULT_ENDPOINT = SpacesServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = SpacesServiceClient.DEFAULT_MTLS_ENDPOINT

    conference_record_path = staticmethod(SpacesServiceClient.conference_record_path)
    parse_conference_record_path = staticmethod(
        SpacesServiceClient.parse_conference_record_path
    )
    space_path = staticmethod(SpacesServiceClient.space_path)
    parse_space_path = staticmethod(SpacesServiceClient.parse_space_path)
    common_billing_account_path = staticmethod(
        SpacesServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        SpacesServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(SpacesServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        SpacesServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        SpacesServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        SpacesServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(SpacesServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        SpacesServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(SpacesServiceClient.common_location_path)
    parse_common_location_path = staticmethod(
        SpacesServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SpacesServiceAsyncClient: The constructed client.
        """
        return SpacesServiceClient.from_service_account_info.__func__(SpacesServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SpacesServiceAsyncClient: The constructed client.
        """
        return SpacesServiceClient.from_service_account_file.__func__(SpacesServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return SpacesServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> SpacesServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            SpacesServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(SpacesServiceClient).get_transport_class, type(SpacesServiceClient)
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, SpacesServiceTransport] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the spaces service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.SpacesServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = SpacesServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_space(
        self,
        request: Optional[Union[service.CreateSpaceRequest, dict]] = None,
        *,
        space: Optional[resource.Space] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resource.Space:
        r"""`Developer
        Preview <https://developers.google.com/workspace/preview>`__.
        Creates a space.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import meet_v2beta

            async def sample_create_space():
                # Create a client
                client = meet_v2beta.SpacesServiceAsyncClient()

                # Initialize request argument(s)
                request = meet_v2beta.CreateSpaceRequest(
                )

                # Make the request
                response = await client.create_space(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.meet_v2beta.types.CreateSpaceRequest, dict]]):
                The request object. Request to create a space.
            space (:class:`google.apps.meet_v2beta.types.Space`):
                Space to be created. As of May 2023,
                the input space can be empty. Later on
                the input space can be non-empty when
                space configuration is introduced.

                This corresponds to the ``space`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.meet_v2beta.types.Space:
                [Developer Preview](\ https://developers.google.com/workspace/preview).
                   Virtual place where conferences are held. Only one
                   active conference can be held in one space at any
                   given time.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([space])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.CreateSpaceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if space is not None:
            request.space = space

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_space,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_space(
        self,
        request: Optional[Union[service.GetSpaceRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resource.Space:
        r"""`Developer
        Preview <https://developers.google.com/workspace/preview>`__.
        Gets a space by ``space_id`` or ``meeting_code``.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import meet_v2beta

            async def sample_get_space():
                # Create a client
                client = meet_v2beta.SpacesServiceAsyncClient()

                # Initialize request argument(s)
                request = meet_v2beta.GetSpaceRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_space(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.meet_v2beta.types.GetSpaceRequest, dict]]):
                The request object. Request to get a space.
            name (:class:`str`):
                Required. Resource name of the space.
                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.meet_v2beta.types.Space:
                [Developer Preview](\ https://developers.google.com/workspace/preview).
                   Virtual place where conferences are held. Only one
                   active conference can be held in one space at any
                   given time.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.GetSpaceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_space,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_space(
        self,
        request: Optional[Union[service.UpdateSpaceRequest, dict]] = None,
        *,
        space: Optional[resource.Space] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resource.Space:
        r"""`Developer
        Preview <https://developers.google.com/workspace/preview>`__.
        Updates a space.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import meet_v2beta

            async def sample_update_space():
                # Create a client
                client = meet_v2beta.SpacesServiceAsyncClient()

                # Initialize request argument(s)
                request = meet_v2beta.UpdateSpaceRequest(
                )

                # Make the request
                response = await client.update_space(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.apps.meet_v2beta.types.UpdateSpaceRequest, dict]]):
                The request object. Request to update a space.
            space (:class:`google.apps.meet_v2beta.types.Space`):
                Required. Space to be updated.
                This corresponds to the ``space`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Optional. Field mask used to specify the fields to be
                updated in the space. If update_mask isn't provided, it
                defaults to '*' and updates all fields provided in the
                request, including deleting fields not set in the
                request.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.apps.meet_v2beta.types.Space:
                [Developer Preview](\ https://developers.google.com/workspace/preview).
                   Virtual place where conferences are held. Only one
                   active conference can be held in one space at any
                   given time.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([space, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.UpdateSpaceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if space is not None:
            request.space = space
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_space,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("space.name", request.space.name),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def end_active_conference(
        self,
        request: Optional[Union[service.EndActiveConferenceRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""`Developer
        Preview <https://developers.google.com/workspace/preview>`__.
        Ends an active conference (if there is one).

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.apps import meet_v2beta

            async def sample_end_active_conference():
                # Create a client
                client = meet_v2beta.SpacesServiceAsyncClient()

                # Initialize request argument(s)
                request = meet_v2beta.EndActiveConferenceRequest(
                    name="name_value",
                )

                # Make the request
                await client.end_active_conference(request=request)

        Args:
            request (Optional[Union[google.apps.meet_v2beta.types.EndActiveConferenceRequest, dict]]):
                The request object. Request to end an ongoing conference
                of a space.
            name (:class:`str`):
                Required. Resource name of the space.
                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.EndActiveConferenceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.end_active_conference,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def __aenter__(self) -> "SpacesServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("SpacesServiceAsyncClient",)
